/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2009 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2009.
 *   author Alberto Romeu aromeu@prodevelop.es 
 *   author Ruben Blanco rblanco@prodevelop.es 
 *   
 */

package es.prodevelop.gvsig.mini.common.impl;

import es.prodevelop.gvsig.mini.geom.Extent;
import es.prodevelop.gvsig.mini.geom.Pixel;
import es.prodevelop.gvsig.mini.utiles.Cancellable;

/**
 * Container class of a single Tile instance.
 * 
 * @author aromeu
 * @author rblanco
 * 
 */
public class Tile {

	public String mURL;
	public int[] tile;
	public Pixel distanceFromCenter;
	public int zoomLevel;
	public String layerName;
	public Cancellable cancellable;
	public Extent extent;

	/**
	 * The constructor
	 * 
	 * @param url
	 *            The url of the tile
	 * @param tile
	 *            The tile x-y
	 * @param distanceFromCenter
	 *            Distance from the center of the screen in pixels
	 * @param zoomLevel
	 *            The zoomLevel of this tile
	 * @param layerName
	 *            The name of the MapRenderer
	 * @param cancellable
	 *            A cancellable instance to check if the tile is valid or not
	 * @param extent
	 * 			  The extent of the tile 
	 */
	public Tile(String url, int[] tile, Pixel distanceFromCenter,
			final int zoomLevel, final String layerName,
			final Cancellable cancellable, final Extent extent) {
		this.mURL = url;
		this.tile = tile;
		this.distanceFromCenter = distanceFromCenter;
		this.zoomLevel = zoomLevel;
		this.layerName = layerName;
		this.cancellable = cancellable;
		this.extent = extent;
	}

	/**
	 * Frees memory
	 */
	public void destroy() {
		this.mURL = null;
		tile = null;
		distanceFromCenter = null;
	}

	public Tile clone() {
		Tile clone = new Tile(this.mURL, this.tile, this.distanceFromCenter,
				this.zoomLevel, this.layerName, cancellable, extent);
		return clone;
	}

	public String getTileString() {
		return new StringBuffer().append(tile[0]).append("_").append(tile[1])
				.append("_").append(zoomLevel).toString();
//		return mURL;
	}
}
