/* gvSIG. Geographic Information System of the Valencian Government
*
* Copyright (C) 2007-2008 Infrastructures and Transports Department
* of the Valencian Government (CIT)
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
* MA  02110-1301, USA.
* 
*/

/*
* Prodevelop, S.L.
* Pza. Juan de Villarrasa, 14 - 5
* Valencia
* Spain
* Tel: +34 963510612
* e-mail: prode@prodevelop.es
* http://www.prodevelop.es
*
*/
package es.prodevelop.gvsig.mini.geom;

/**
 * An abstraction of an entity in the real world
 * @author Alberto Romeu Carrasco (aromeu@prodevelop.es)
 */
public class Feature {    
    
    private IGeometry geometry;
    private boolean visible = true;    

    /**
     * The constructor.
     * @param geometry A geometry
     */
    public Feature(final IGeometry geometry) {
        this.geometry = geometry;
    }

    /**
     * 
     * @return true if the feature is visible (is drawn on the screen)
     */
    public boolean isVisible() {
        return visible;
    }

    /**
     * Sets the feature and its geometry visible or not
     * @param visible
     */
    public void setVisible(final boolean visible) {
        this.visible = visible;
        if (this.geometry != null) {
            this.geometry.setVisible(visible);
        }
    }

    /**
     * 
     * @return THe geometry
     */
    public IGeometry getGeometry() {
        return geometry;
    }

    /**
     * Sets the geometry
     * @param geometry A geometry
     */
    public void setGeometry(final IGeometry geometry) {
        this.geometry = geometry;
    }

    /**
     * Clears this instance
     */
    public void destroy() {
        try {            
            this.geometry.destroy();
            this.geometry = null;            
        } catch (final Exception e) {
            e.printStackTrace();
        }
    }
}
