/* gvSIG. Geographic Information System of the Valencian Government
*
* Copyright (C) 2007-2008 Infrastructures and Transports Department
* of the Valencian Government (CIT)
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
* MA  02110-1301, USA.
* 
*/

/*
* Prodevelop, S.L.
* Pza. Juan de Villarrasa, 14 - 5
* Valencia
* Spain
* Tel: +34 963510612
* e-mail: prode@prodevelop.es
* http://www.prodevelop.es
*
*/

package es.prodevelop.gvsig.mini.namefinder;

import es.prodevelop.gvsig.mini.geom.Extent;
import es.prodevelop.gvsig.mini.geom.Pixel;
import es.prodevelop.gvsig.mini.geom.Point;

/**
 * A container with the properties of a named tag in the namefinder response.
 * A named contains the lon lat coordinates received from the service that are immutable
 * and an attribute that contains the projectedCoordinates in the projection
 * of the base layer
 * @author Alberto Romeu Carrasco (aromeu@prodevelop.es)
 */
public class Named extends Point {
    
    /**
     * node, way or relation
     */
    public String type;
    
    /**
     * The id of the named
     */
    public int id;    
    
    /**
     * The name
     */
    public String name;
    
    /**
     * The category
     */
    public String category;
    
    /**
     * The info
     */
    public String info;
    
    /**
     * The rank of the named, that is, how important is.
     */     
    public int rank;
    
    /**
     * Where is the named
     */
    public String isIn;
    
    /**
     * The distance from the parent named
     */
    public double distance;
    
    /**
     * The truncated distance
     */
    public double approxDistance;
    
    /**
     * The angle 
     */
    public double directionAngle;
    
    /**
     * The OSM zoom to fit this named
     */
    public int zoom;
    
    /**
     * The full description
     */
    public String description;
    
    /**
     * Contains a named instance corresponding to the place tag of the response of the service
     */
    public Named place;
    
    /**
     * An array of nearestplaces
     */
    public Named[] nearestPlaces;
    
    /**
     * The coordinates of the named in the current projection of the base layer of the map
     */
    public Point projectedCoordinates;
    
    /**
     * The constructor
     * @param x lon
     * @param y lat
     */
    public Named(double x, double y) {        
        super(x, y);
        
    }
    
    /**
     * Clears the named
     */
    public void destroy() {
        try {
            type = null;    
            name = null;
            category = null;
            info = null;    
            isIn = null;        
            description = null;
            try {
                place.destroy();
            } catch (final Exception e) {
                e.printStackTrace();
            } finally {
                place = null;
            }
            final int size = nearestPlaces.length;
            final Named[] list = this.nearestPlaces;
            for (int i = 0; i<size; i++) {
                try {
                    list[i].destroy();
                } catch (final Exception e) {
                    e.printStackTrace();
                }                
            }
    
            nearestPlaces = null;;
            projectedCoordinates = null;
        } catch (final Exception e) {
            e.printStackTrace();
        }
    }
    
    /**
     * 
     * @return The name
     */
    public String toString() {
        if (this.name != null)
            return this.name;
        return "";
    }
    
    /**
     * Sets lat lon coordinates
     * @param lon
     * @param lat
     * @param fillProjectedCoordinates If true, the projectedCoordinates will be the same
     * as lat lon coordinates
     */
    public void setLonLat(final double lon, final double lat, final boolean fillProjectedCoordinates) {
        this.setX(lon);
        this.setY(lat);        
        if(fillProjectedCoordinates) {
            this.projectedCoordinates = new Point(lon, lat);
        }
    }
    
    public double[] getLonLat() {
        return new double[]{getX(), getY()};
    }
    
    public double getLon() {
        return this.getX();
    }
    
    public double getLat() {
        return this.getY();
    }  
    
    /**
     * Sets the projectedCoordinates
     * @param coords
     */
    public void setCoordinates(final Point[] coords) {
        try {
            this.projectedCoordinates = coords[0];
        } catch (final Exception e) {
            e.printStackTrace();
        }
    }
    
    /**
     * Sets the projectedCoordinates
     * @param coords
     */
    public void setCoordinates(final double[] coords) {
        try {
            this.projectedCoordinates.setCoordinates(coords);
        } catch (final Exception e) {
            e.printStackTrace();
        }
    }
    
    /**
     * Clones the projectedCoordinates
     * @return
     */
    public Object clone() {
        return this.projectedCoordinates.clone();
    }
}
